/**
 *	fs_sem  --  semaphore client test program
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Jan.1987.  swevent
 *	K.Furukawa  Sep.1993.  he_echo
 *	K.Furukawa  Apr.16.1994.  fs_sem
 *	Modifications
 *
 *      This program releases (posts) a semaphore and waits for another 
 *	semaphore, which will be released by the server program.
 *	This will be the typical example for the semaphore server-client 
 *	model.
 *	With the loop option fs_sem loops to measure average response.
 *	See argument() for options.
**/

char	what_fssem[] = 
"@(#)fs_sem v1.0.1 semaphore client, K.Furukawa, Jan.1987-Apr.1994";

#include <stdio.h>
#include "semsub.h"

#define  PORTNUMB	11069		/* service port number */

#define  OPT		FS_OALL		/* error message option */

int	sd;				/* semaphore descriptor */
int	sd2;				/* second semaphore descriptor */
char	*nm;				/* program name holder */
int	portnum;			/* port number */
int	loopcount;			/* count for loop option */
int	sleeptime;			/* sleep time for interval option */
void	errormsg();

/**
 *  main
 *	Main prepares semaphore
**/
void
 main(argc, argv)
int	argc;
char	*argv[];
{
    void argument();
    void client();

    nm = argv[0];				/* set program name */
    portnum = PORTNUMB;				/* default port */
    loopcount = 0;				/* no loop option by default */
    sleeptime = 0;				/* no sleep by default */

    argument(argc, argv);			/* command line argument */

    sd = fs_attach( portnum, OPT );		/* attach to a semaphore */
    if ( sd < 0 )  exit( 1 );
    sd2 = fs_attach( portnum+1, OPT );		/* attach to a semaphore */
    if ( sd2 < 0 )  exit( 1 );

    client();
}

/**
 *  client
 *	Client release a semaphore, which the server waits for, and 
 *	waits for another semaphore, which the server releases.
**/
void
 client()
{
    long lines = 0;				/* communication counter */
    int	i;					/* loop counter */
    char * buf[BUFSIZ];

    if ( !loopcount )  {			/* no loop option */

	write( 2, "type return\n", 12 );

	while( fgets( buf, BUFSIZ, stdin ) != NULL )  {

	    if( fs_release(sd, OPT) == FS_SUCCESS )  {
						/* post semaphore for server */
		write( 2, "post\n", 5 );

		if( fs_wait(sd2, OPT) == FS_SUCCESS )  {
		    				/* wait semaphore */
		    write( 2, "wait\n", 5 );
		    write( 2, "type return\n", 12 );
		}
		else
		    errormsg( "semaphore wait failure" );
	    }  else  {
		errormsg( "semaphore post failure" );
		exit( 1 );
	    }
	    lines++;
	}
    
    }  else  {

	while( lines < loopcount )  {		/* loop specified times */
	    if( fs_release(sd, OPT) == FS_SUCCESS )  {
						/* post semaphore for server */
		if ( fs_wait(sd2, OPT) == FS_SUCCESS )
		    				/* wait the server */
		    ;
		else
		    errormsg( "semaphore wait failure" );
	    }  else  {
		errormsg( "semaphore post failure" );
		exit( 1 );
	    }
	    lines++;
	    if ( sleeptime )  sleep( sleeptime );
	}

    }

    fprintf( stderr, "  %ld times\n", lines );
    fs_detach( portnum, OPT );			/* detach from semaphore */
    fs_detach( portnum+1, OPT );		/* detach from semaphore */
}


/*
 *  errormsg
 *	print error message
 */
void
 errormsg( msg )
char * msg;
{
#ifndef OSK
	perror( nm );
#else
	extern int errno;
	prerr(0,errno);
	fprintf(stderr,"%s\n", nm);
#endif
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
void
argument(argc, argv)
int	argc;
char	*argv[];
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "p:P:l:L:i:I:hH")) != EOF)
	    switch ( c )  {
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 'i':
	    case 'I':				/* sleep interval time */
		sleeptime = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( argc == optind )  return;

usage:
	fprintf(stderr, "%s\n", what_fssem );
	fprintf(stderr, "Usage:  fs_sem  <option>\n" );
	fprintf(stderr, " option:\n" );
	fprintf(stderr, "          -pX  use semaphore number X\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: fs_sem\n" );
	fprintf(stderr, "          fs_sem -p2000 -l10000\n" );
	exit( 1 );
}
