/**
 *	fs_semd  --  semaphore server test program
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Jan.1987.  swevented
 *	K.Furukawa  Sep.1993.  he_echod
 *	K.Furukawa  Apr.16.1994.  fs_semd
 *	Modifications
 *
 *      This program waits for a semaphore, which is released by the
 *	client program, and releases (posts) another semaphore.
 *	This will be the typical example for the semaphore server-client 
 *	model.
 *	See argument() for options.
**/

char	what_fssemd[] = 
"@(#)fs_semd v1.0.1 semaphore server, K.Furukawa, Jan.1987-Apr.1994";

#include <stdio.h>
#include "semsub.h"

#define  PORTNUMB	11069		/* service port number */

#define  OPT		FS_OALL		/* error message option */

int	sd;				/* semaphore descriptor */
int	sd2;				/* second semaphore descriptor */
char	*nm;				/* program name holder */
int	portnum;			/* port number */
void	errormsg();

/**
 *  main
 *	Main prepares semaphore
**/
void
 main(argc, argv)
int	argc;
char	*argv[];
{
    int ret;
    void argument();
    void server();

    nm = argv[0];				/* set program name */
    portnum = PORTNUMB;				/* default port */

    argument(argc, argv);			/* command line argument */

    sd = fs_create( portnum, FS_POTHER, FS_OERR );  /* create a semaphore */
    if ( sd != FS_SUCCESS && sd != FS_EXIST )  exit( 1 );
    sd = fs_attach( portnum, OPT );		/* attach to a semaphore */
    if ( sd < 0 )  exit( 1 );
    ret = fs_reset( sd, OPT );			/* reset the semaphore */
    if ( ret != FS_SUCCESS )  exit( 1 );
    sd2 = fs_create( portnum+1, FS_POTHER, FS_OERR ); /* create a semaphore */
    if ( sd2 != FS_SUCCESS && sd2 != FS_EXIST )  exit( 1 );
    sd2 = fs_attach( portnum+1, OPT );		/* attach to a semaphore */
    if ( sd2 < 0 )  exit( 1 );
    ret = fs_reset( sd2, OPT );			/* reset the semaphore */
    if ( ret != FS_SUCCESS )  exit( 1 );

    server();
}

/**
 *  server
 *	Server waits for a semaphore, which the client releases, and 
 *	releases another semaphore, which the client will wait for.
**/
void
 server()
{
    long lines = 0;				/* communication counter */
    int	i;					/* loop counter */
    char * buf[BUFSIZ];

    while ( 1 )  {
	if( fs_wait(sd, OPT) == FS_SUCCESS )  {  /* wait semaphore */
	    if( fs_release(sd2, OPT) == FS_SUCCESS )
	    					/* release another semaphore */
		;
	    else
		errormsg( "semaphore post failure" );
	}  else  {
	    errormsg( "semaphore wait failure" );
	    exit( 1 );
	}
	lines++;
    }
    
}


/*
 *  errormsg
 *	print error message
 */
void
 errormsg( msg )
char * msg;
{
#ifndef OSK
	perror( nm );
#else
	extern int errno;
	prerr(0,errno);
	fprintf(stderr,"%s\n", nm);
#endif
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
void
argument(argc, argv)
int	argc;
char	*argv[];
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "p:P:hH")) != EOF)
	    switch ( c )  {
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( argc == optind )  return;

usage:
	fprintf(stderr, "%s\n", what_fssemd );
	fprintf(stderr, "Usage:  fs_semd  <option>\n" );
	fprintf(stderr, " option:\n" );
	fprintf(stderr, "          -pX  use semaphore number X\n" );
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: fs_semd\n" );
	fprintf(stderr, "          fs_semd -p2000\n" );
	exit( 1 );
}
