/**
 *	he_echo  --  hecom client test program
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Sep.15.1993. hecomclient based on s2_recho v1.2.0
 *	Modifications
 *	K.Furukawa  Sep.17.1993. name changed from hecomclient to he_echo
 *
 *      This program sends messages typed on the terminal and receives
 *	the echoed reply from hecomserver program.  This will be the typical
 *	example for the hecom server-client model.
 *	With the loop option he_echo loops to measure average response.
 *	See argument() for options.
**/

/**
 *	recho  --  remote echo program
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Nov.9.1988.  (rwrite)
 *      K.F  Apr.13.1989.  converted from rwrite (recho)
 *	K.F  Jun.17.1992.  recho v3.4.4
 *	K.F  Oct.2.1992.   s2_recho using s2_xxx routines
 *	Modifications
 *	K.F  Nov.26.1992.  use getopt routine
 *	K.F  Jul.12.1993.  use fgets instead of gets for os9
 *
 *	This program sends messages typed on the terminal and receives 
 *	the echoed reply from s2_rechod program.  This will be the typical
 *	example for the TCP|UDP/IP server-client model.  Modifying
 *	{tcp|tcp}server() of rechod, recho and rechod function as any 
 *	information server.
 *	With the loop option Recho loops to measure average response.
 *	See argument() for options.
**/

char	what[] = 
"@(#)he_echo.c v1.1.0 hecom echo client, K.Furukawa, Nov.1988-Sep.1993";

#include <stdio.h>

#define  PORTNUMB	11061		/* service port number */
#define  BUFFERSIZE	16384		/* communication buffer size */

#define  OPT		3		/* error message option */

char	buf[BUFFERSIZE];		/* i/o buffer */
int	sd;				/* file descriptor for socket com. */
char	*nm;				/* program name holder */
int	portnum;			/* port number */
int	loopcount;			/* count for loop option */
int	loopsize;			/* buffer size for loop option */
int	sleeptime;			/* sleep time for interval option */
void	errormsg();

/**
 *  main
 *	Main prepares communication line.
**/
void
 main(argc, argv)
int	argc;
char	*argv[];
{
    void argument();
    void client();

    nm = argv[0];				/* set program name */
    portnum = 0;				/* port number */
    loopcount = 0;				/* no loop option by default */
    loopsize = 1;				/* size 1 by default */
    sleeptime = 0;				/* no sleep by default */

    argument(argc, argv);			/* command line argument */

    if ( !portnum )  {				/* port number not specified */
	portnum = PORTNUMB;			/* default port */
    }

    sd = he_open( portnum, BUFFERSIZE, OPT );
    if ( sd<0 )  exit( 1 );			/* connect to server */

    client();
}

/**
 *  client
 *	Client reads standard input, sends it to the server, receives
 *	its reply and displays it.
**/
void
 client()
{
    int	bufcnt = 0, bufcnt2;			/* buffer length */
    long lines = 0;				/* communication counter */
    long bytes = 0;				/* communication counter */
    int	i;					/* loop counter */

    if ( !loopcount )  {			/* no loop option */

	while( fgets( buf, BUFFERSIZE, stdin ) != NULL )  {
	    bufcnt = strlen( buf );		/* read standard input */
	    if( he_send(buf, bufcnt, OPT) == bufcnt )  {
						/* send to the server */
		if( (bufcnt2 = he_receive(buf, BUFFERSIZE, OPT)) > 0 )
		    				/* receive from the server */
		    write( 2, buf, bufcnt2 );
		if (bufcnt != bufcnt2)
		    errormsg( "send/receive size different" );
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt2;
	}
    
    }  else  {

	char  *c = buf;				/* buffer pointer */

	for ( i = 0 ; i < loopsize ; i++ )  *c++ = 'a';
	bufcnt = loopsize;

	while( lines < loopcount )  {		/* loop specified times */
	    if( he_send(buf, bufcnt, OPT) == bufcnt )  {
						/* send to the server */
		if ((bufcnt2 = he_receive(buf, BUFFERSIZE, 
					  OPT)) != bufcnt) {
		    				/* receive from the server */
		    errormsg( "send/receive size different" );
		}
	    }  else  {
		errormsg( "send size different" );
		exit( 1 );
	    }
	    lines++;
	    bytes += bufcnt2;
	    if ( sleeptime )  sleep( sleeptime );
	}

    }

    fprintf( stderr, "  %ld lines %ld bytes\n", lines, bytes );
    he_close( OPT );
}


/*
 *  errormsg
 *	print error message
 */
void
 errormsg( msg )
char * msg;
{
#ifndef OSK
	perror( nm );
#else
	extern int errno;
	prerr(0,errno);
	fprintf(stderr,"%s\n", nm);
#endif
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
void
argument(argc, argv)
int	argc;
char	*argv[];
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "p:P:l:L:s:S:i:I:hH")) != EOF)
	    switch ( c )  {
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case 'l':
	    case 'L':				/* loop count */
		loopcount = atoi( optarg );
		break;
	    case 's':
	    case 'S':				/* loop buffer size */
		loopsize = atoi( optarg );
		break;
	    case 'i':
	    case 'I':				/* sleep interval time */
		sleeptime = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( argc == optind )  return;

usage:
	fprintf(stderr, "%s\n", what );
	fprintf(stderr, "Usage:  he_echo  <option>\n" );
	fprintf(stderr, " option:\n" );
	fprintf(stderr, "          -pX  use service port number X\n" );
	fprintf(stderr, "          -lY  loop Y times\n" );
	fprintf(stderr, "          -sZ  send Z bytes at a time (default=1)\n");
	fprintf(stderr, "          -iW  sleep W seconds interval\n");
	fprintf(stderr, "          -h   display help message\n" );
	fprintf(stderr, " example: he_echo\n" );
	fprintf(stderr, "          he_echo -p2000 -l100 -s10\n" );
	exit( 1 );
}
