/**
 *	semsub.h  --  code definition for semsub
 *		Kazuro Furukawa
 *
 *	K.Furukawa  Aug.14.1993.(2)
 *	K.Furukawa  May.8.1994.      posix semaphore
 *	K.Furukawa  Aug.17.1997.     check _OSK and OSK
 *
 * @(#)semsub.h v1.1.2  definition for semsub, K.Furukawa, Aug.1993-Aug.1997.
 */

#ifndef __SEMSUB_H
#define __SEMSUB_H

#ifdef _OSK
#ifndef OSK
#define OSK
#endif
#endif

#ifndef OSK  /* for unix */
#ifndef posix  /* for unix */
#ifndef unix
#define unix 1
#endif /* unix */
#endif /* not posix */
#ifndef __SHMSUB_H
#include <sys/types.h>				/* types */
#include <sys/stat.h>				/* permissions */
#endif /* not __SHMSUB_H */
#endif /* not OSK */

#ifdef OSK
/* #include <module.h>				/* module header */
#endif /* OSK */

/* error codes */

#define FS_SUCCESS	0		/* success */
#define FS_NORES	-1		/* no new semaphore available */
#define FS_OTHER	-2		/* other system call failure */
#define FS_EXIST	-3		/* semaphore existent */
#define FS_NOEXIST	-4		/* semaphore non existent */

/* error/information message codes */
/* currently no difference among OERR, OINFO, OALL */

#define FS_ONONE	0		/* no message */
#define FS_OERR		1		/* error message */
#define FS_OINFO	2		/* information message */
#define FS_OALL		3		/* all message */

/* permission codes */

#ifdef unix
#define FS_POWNER	(S_IRWXU|S_IRGRP)		/* owner+group read */
#define FS_PGROUP	(FS_POWNER|S_IRWXG|S_IROTH)	/* +world read */
#define FS_POTHER	(FS_PGROUP|S_IRWXO)		/* +world any */
#endif /* unix */

#ifdef OSK
#define FS_POWNER	0				/* not used */
#define FS_PGROUP	0				/* not used */
#define FS_POTHER	0				/* not used */
#endif /* OSK */

#define FS_PWORLD	FS_POTHER
#define FS_PALL		FS_POTHER

/* functions */

int fs_check();				/* in semsub.c */
int fs_info();
int fs_create();
int fs_attach();
int fs_release();
int fs_wait();
int fs_remove();

#endif /* __SEMSUB_H */
